//---------------------------------------------------------------------------
#ifndef TEFSCUnitH
#define TEFSCUnitH

#include <fstream.h>


const UINT UNIT_ON_PLANET =   0x00000001;

const UINT UNIT_SENTRY    =   0x00000004;

const UINT UNIT_IS_CARGO  =   0x00000040;
const UINT UNIT_ON_LOAN   =   0x00000080;

const UINT UNIT_DELETED   =   0x80000000;

const MAX_CARGO = 4;

class TEFSCUnit {
private:
  short FPIdx;      // Planet index of this unit's location
  short Fx, Fy;     // Coordinates of this unit's location;
                    //the hex coordinates if on the planet surface, or
                    //the tile coordinates on the Star Map grid if in space
  char  FLoyalty;   // Unit Loyalty, as a percent

protected:

public:
  char  Owner;      // Faction to which this unit belongs (currently)
  char  PrevOwner;  // Faction to which this unit normally belongs
  char  Type;       // Index to Unit Table entry describing this unit's characteristics
  char  tLvl;       // Secondary index used to look up this unit's characteristics 
  short MoveType;   // (Not saved) Enumerated movement type, e.g., F, W, etc.?
  short Orders;     // Enumerated orders: Move, Sentry, etc.?
  char  Experience; // As a level, e.g., 0, 1 or 2?
  char  MovePoints; // Current Movement Points remaining(?)
  union {
    char  Res;      // Resource type or Relic number
    char  RelicType;
  } ResRelic;
  short Amount;     // Number of resources, if this unit is a resource pod(?)
  char  Health;     // Current Health, as a percent
  char  Sect;       // Enumeration specifying which Sect this unit belongs to
  int   UnitNo;     // Unique ID number for this unit?
  UINT  Flags;      // Current status flags for this unit
  char  UsedUnitType; // Indicies into the Unit Table for the unit type and
  char  UsedUnittLvl; //tech level of the unit from which this unit is/was built
  char  Camo;       // The infamous "camo" value for this unit
  UCHAR DestX;      // Coordinates for the currently set movement destination
  UCHAR DestY;      //for this unit?
  int   MoveCost;   // (Not saved) Unknown
  UCHAR tFlags;     // (Not saved) Unknown
  char  AIOrders;   // Unknown
  char  AIData;     // Unknown
  short AIData2;    // Unknown
  UCHAR WaitLevel;  // Unknown - probably dictates how/when this unit comes up
                    //in the rotation of units while issuing orders

  TEFSCUnit*  pNext;
  TEFSCUnit*  pPrev;
  TEFSCUnit*  pNIG;
  TEFSCUnit*  pPIG;

  TEFSCUnit*  Cargo[MAX_CARGO];

  short GroupEndMark; // Used when simply loading/re-saving files without
                      //manipulating groups/stacks/cargo (i.e., no changes
                      //to unit list linkages)

  TEFSCUnit();
  ~TEFSCUnit();

  void SetPosition(short pIdx, short x, short y)
    { FPIdx = pIdx; Fx = x; Fy = y; }
  short X() { return Fx; }
  short Y() { return Fy; }
  short PIdx() { return FPIdx; }

  bool OnPlanet() { return Flags & UNIT_ON_PLANET; }
  bool InSpace()  { return !(Flags & UNIT_ON_PLANET); }
  bool Sentried() { return Flags & UNIT_SENTRY; }
  bool IsCargo()  { return Flags & UNIT_IS_CARGO; }
  bool Deleted()  { return Flags & UNIT_DELETED; }
  bool Loaned()   { return ((-1 != PrevOwner) && (Owner != PrevOwner)); }

  void SetLoyalty(int loyalty) { FLoyalty = loyalty; }
  int Loyalty() { return FLoyalty; }

  int Save(ofstream* outfile);
  int Load(ifstream* infile, bool saveGame, int saveVersion, int origVersion);
  
};
//---------------------------------------------------------------------------
#endif
