//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "TStarMapView.h"
#include "TEFSMain.h"
#include "TGalaxyModule.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TStarMapView *StarMapView;
//---------------------------------------------------------------------------
__fastcall TStarMapView::TStarMapView(TComponent* Owner)
  : TForm(Owner)
{
  TileWidth = STAR_MAP_TILE_WIDTH;
  TileHeight = STAR_MAP_TILE_HEIGHT;

  // Change these initializers for BOTH constructors
  ShowPlanetNames = true;
  ShowPlanets = true;
  ShowJumpGates = true;
  ShowUnits = true;
  ShowMapTileValues = false;
  SupressZeros = true;

  // Change these initializers for BOTH constructors
  ShowPlanetNamesAction->Checked = ShowPlanetNames;
  ShowPlanetsAction->Checked = ShowPlanets;
  ShowJumpGatesAction->Checked = ShowJumpGates;
  ShowMapTileValuesAction->Checked = ShowMapTileValues;
  ShowUnitsAction->Checked = ShowUnits;
}
//---------------------------------------------------------------------------
__fastcall TStarMapView::TStarMapView(TComponent* Owner, TEFSCGalaxy* galaxy)
  : TForm(Owner)
{
  TileWidth = STAR_MAP_TILE_WIDTH;
  TileHeight = STAR_MAP_TILE_HEIGHT;
  Galaxy = galaxy;

  // Change these initializers for BOTH constructors
  ShowPlanetNames = true;
  ShowPlanets = true;
  ShowJumpGates = true;
  ShowUnits = true;
  ShowMapTileValues = false;
  SupressZeros = true;

  // Change these initializers for BOTH constructors
  ShowPlanetNamesAction->Checked = ShowPlanetNames;
  ShowPlanetsAction->Checked = ShowPlanets;
  ShowJumpGatesAction->Checked = ShowJumpGates;
  ShowUnitsAction->Checked = ShowUnits;
  ShowMapTileValuesAction->Checked = ShowMapTileValues;
  SZMapTileValuesAction->Checked = SupressZeros;

  SelTileX = SelTileY = 0;
  CursorColor = clLime;
  UnitSlotColor = clMaroon;
}
//---------------------------------------------------------------------------

void __fastcall TStarMapView::FormCreate(TObject *Sender)
{
  Top = 0;
  Left = 0;
  Width = EFSMain->ClientWidth / 2;
  Height = EFSMain->ClientHeight - EFSMain->EFSMainToolBar->Height
            - EFSMain->EFSMainStatusBar->Height - 4;
  Space->Width = TileWidth * STAR_MAP_WIDTH;
  Space->Height = TileHeight * STAR_MAP_HEIGHT;
  /*
  for (int i=0; i<STAR_MAP_WIDTH; i++) {
    for (int j=0; j<STAR_MAP_HEIGHT; j++) {
      TileCaptions[i][j] = new TLabel(this);

      TileCaptions[i][j]->Left = TileWidth*i;
      TileCaptions[i][j]->Top = TileHeight*j;
      TileCaptions[i][j]->Alignment = taCenter;
      TileCaptions[i][j]->Color = clBlack;

      TileCaptions[i][j]->Font->Name = "Small Fonts";
      TileCaptions[i][j]->Font->Color = clWhite;
      TileCaptions[i][j]->Font->Size = 6;

      TileCaptions[i][j]->Caption = Galaxy->MapTiles[i][j];
      TileCaptions[i][j]->AutoSize = false;
      TileCaptions[i][j]->Width = TileWidth;

      TileCaptions[i][j]->Parent = this;
    }
  }
  */
}
//---------------------------------------------------------------------------

void __fastcall TStarMapView::FormPaint(TObject *Sender)
{
  UpdateView();
}
//---------------------------------------------------------------------------

void __fastcall TStarMapView::FormClose(TObject *Sender,
      TCloseAction &Action)
{
  Action = caFree;
  StarMapView = NULL;
}
//---------------------------------------------------------------------------

void __fastcall TStarMapView::ShowPlanetsActionExecute(TObject *Sender)
{
  ShowPlanets = !ShowPlanets;
  ShowPlanetsAction->Checked = ShowPlanets;
  UpdateView();
}
//---------------------------------------------------------------------------

void __fastcall TStarMapView::ShowPlanetNamesActionExecute(TObject *Sender)
{
  ShowPlanetNames = !ShowPlanetNames;
  ShowPlanetNamesAction->Checked = ShowPlanetNames;
  UpdateView();
}
//---------------------------------------------------------------------------

void __fastcall TStarMapView::ShowJumpGatesActionExecute(TObject *Sender)
{
  ShowJumpGates = !ShowJumpGates;
  ShowJumpGatesAction->Checked = ShowJumpGates;
  UpdateView();
}
//---------------------------------------------------------------------------

void __fastcall TStarMapView::ShowUnitsActionExecute(TObject *Sender)
{
  ShowUnits = !ShowUnits;
  ShowUnitsAction->Checked = ShowUnits;
  UpdateView();
}
//---------------------------------------------------------------------------

void __fastcall TStarMapView::ShowMapTileValuesActionExecute(
      TObject *Sender)
{
  ShowMapTileValues = !ShowMapTileValues;
  ShowMapTileValuesAction->Checked = ShowMapTileValues;
  UpdateView();;
}
//---------------------------------------------------------------------------

void __fastcall TStarMapView::SZMapTileValuesActionExecute(
      TObject *Sender)
{
  SupressZeros = !SupressZeros;
  SZMapTileValuesAction->Checked = SupressZeros;
  UpdateView();;
}
//---------------------------------------------------------------------------

void TStarMapView::UpdateView()
{
  TCanvas* SC = Space->Canvas;
  SC->Brush->Color = clBlack;
  SC->FillRect(Windows::TRect(0, 0, Space->Width, Space->Height));

  if (ShowMapTileValues) {
    SC->Font->Name = "Small Fonts";
    SC->Font->Color = clYellow;
    SC->Font->Size = 6;

    for (int i=0; i<STAR_MAP_WIDTH; i++) {
      for (int j=0; j<STAR_MAP_HEIGHT; j++) {
        if (!SupressZeros || (Galaxy->MapTiles[i][j] != 0)) {
          String MapTileValue = Galaxy->MapTiles[i][j];
          int X = i * TileWidth + TileWidth/2;
          int Y = j * TileHeight;
          int ValueOffset = SC->TextWidth(MapTileValue) / 2 + 1;
          SC->TextOut(X - ValueOffset, Y + 2, MapTileValue);
        }
      }
    }
  }
  
  if (ShowJumpGates) {
    for (int i=0; i<Galaxy->JumpGateCount; i++) if (Galaxy->JumpGates[i]) {
      SC->Pen->Color = clOlive;
      SC->MoveTo(Galaxy->JumpGates[i]->x1 * TileWidth + TileWidth/2, Galaxy->JumpGates[i]->y1 * TileHeight + TileHeight/2);
      SC->LineTo(Galaxy->JumpGates[i]->x2 * TileWidth + TileWidth/2, Galaxy->JumpGates[i]->y2 * TileHeight + TileHeight/2);
    }
  }
  
  for (int i=0; i<Galaxy->PlanetCount; i++) if (Galaxy->Planets[i]) {
    int X, Y;
    X = Galaxy->Planets[i]->x * TileWidth + TileWidth/2;
    Y = Galaxy->Planets[i]->y * TileHeight + TileHeight/2;
    if (ShowPlanets) {
      if (GalaxyModule->PlanetType->FindKey(OPENARRAY(TVarRec, (Galaxy->Planets[i]->TileSetType + 1)))) {
        SC->Brush->Color = TColor(GalaxyModule->Color->AsInteger);
      }
      else SC->Brush->Color = clAqua;
      SC->Ellipse(X - TileWidth/4, Y - TileHeight/4, X + TileWidth/4, Y + TileHeight/4);

      int X = Galaxy->Planets[i]->x;
      int Y = Galaxy->Planets[i]->y;
      OutlineTile(X-1, Y-1, UnitSlotColor, pmCopy);
      OutlineTile(X, Y-1, UnitSlotColor, pmCopy);
      OutlineTile(X+1, Y-1, UnitSlotColor, pmCopy);
      OutlineTile(X-1, Y, UnitSlotColor, pmCopy);
      OutlineTile(X+1, Y, UnitSlotColor, pmCopy);
      OutlineTile(X-1, Y+1, UnitSlotColor, pmCopy);
      OutlineTile(X, Y+1, UnitSlotColor, pmCopy);
      OutlineTile(X+1, Y+1, UnitSlotColor, pmCopy);

    }

    if (ShowPlanetNames) {
      String PN = Galaxy->Planets[i]->Name;
      int NameOffset = SC->TextWidth(PN) / 2;
      SC->Brush->Color = clBlack;
      if (GalaxyModule->PlanetTable->FindKey(OPENARRAY(TVarRec, (i)))
           && (i >= 0)) {
        SC->Font->Color = static_cast<TColor>(GalaxyModule->PlanetTableOwnerColorLU->AsInteger);
      }
      else SC->Font->Color = clGray;

      SC->Font->Name = "MS Sans Serif";
      SC->Font->Size = 8;
      SC->TextOut(X - NameOffset, Y + TileHeight/4, PN);
    }
  }

  if (ShowUnits) {
    list<TEFSCUnit*>::iterator unit;
    for (unit = Galaxy->Units.begin(); unit != Galaxy->Units.end(); unit++) {
      if ((*unit)->InSpace()) {
        int X, Y;
        X = (*unit)->X() * TileWidth + TileWidth/2;
        Y = (*unit)->Y() * TileHeight + TileHeight/2;
        Windows::TRect Rect(X - TileWidth/4, Y - TileHeight/4, X + TileWidth/4, Y + TileHeight/4);

        short owner;
        if ((*unit)->Loaned()) owner = (*unit)->PrevOwner;
        else owner = (*unit)->Owner;

        if (GalaxyModule->FactionTable->FindKey(OPENARRAY(TVarRec, (owner)))
             && (owner >= 0)) {
          SC->Brush->Color = TColor(GalaxyModule->FactionTableColor->AsInteger);
        }
        else SC->Brush->Color = clBlack;

        SC->FillRect(Rect);
        SC->Brush->Color = clSilver;
        SC->FrameRect(Rect);
      }
    }
  }
  OutlineTile(SelTileX, SelTileY, CursorColor, pmXor);
}
//---------------------------------------------------------------------------

void __fastcall TStarMapView::SpaceMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
  int tx, ty;
  MouseToTile(X, Y, tx, ty);
  String InfoText = String("Mouse: ") + String(X) + String(", ") + String(Y);
  InfoText = InfoText + String("  Tile: ") + String(tx) + String(", ") + String(ty);
  StatusBar->SimpleText = InfoText;

  OutlineTile(SelTileX, SelTileY, CursorColor, pmXor);
  SelTileX = tx;
  SelTileY = ty;
  OutlineTile(SelTileX, SelTileY, CursorColor, pmXor);

}
//---------------------------------------------------------------------------

void TStarMapView::MouseToTile(int x, int y, int& TileX, int& TileY)
{
  TileX = x / TileWidth;
  TileY = y / TileHeight;
}
//---------------------------------------------------------------------------

void TStarMapView::OutlineTile(int x, int y, TColor color, TPenMode penMode)
{
  TCanvas* SC = Space->Canvas;
  TPenMode SavePenMode = SC->Pen->Mode;
  TColor SavePenColor = SC->Pen->Color;
  SC->Pen->Mode = penMode;
  SC->Pen->Color = color;
  TPoint Corners[5];
  Corners[0] = TPoint(x * TileWidth, y * TileHeight);
  Corners[1] = TPoint(x * TileWidth + TileWidth - 1, y * TileHeight);
  Corners[2] = TPoint(x * TileWidth + TileWidth - 1, y * TileHeight + TileHeight - 1);
  Corners[3] = TPoint(x * TileWidth, y * TileHeight + TileHeight - 1);
  Corners[4] = TPoint(x * TileWidth, y * TileHeight);
  SC->Polyline(Corners, 4);
  SC->Pen->Mode = SavePenMode;
  SC->Pen->Color = SavePenColor;
}
//---------------------------------------------------------------------------

